c	For the description see the subroutine roche
c	based on roche7.f
	implicit double precision (a-h,o-z)
	parameter(ndim1=2001,ndim2=2001)
	dimension x(ndim1),y(ndim2),rosu(ndim1,ndim2),rotem(ndim1,ndim2)
c     p	,rogrv(ndim1,ndim2),rocos(ndim1,ndim2)
        pi=3.1415926535897931d0
	write(*,*)'             Program Roche'
	write(*,*)' input: roche.in'
	write(*,*)' output: roche.out'
	open(2,file='roche.in',status='old')
	read(2,*)
	read(2,*)nbod1,nbod2,x1,x2,y1,y2
	read(2,*)
        read(2,*)q,ff,xcomp,ipsc
        read(2,*)
        read(2,*)tstar,rstar,dgst,irrst,albst,htst,htsta
        read(2,*)
        read(2,*)tempcp,rcp,dgcp,irrcp,albcp,htcp,htcpa
        close(2)
        rsol=6.9599d10
        rjup=7.149e9
        x1=x1*rsol
        x2=x2*rsol
        y1=y1*rsol
        y2=y2*rsol        
	xcomp=xcomp*rsol
	rstar=rstar*rsol
	rcp=rcp*rsol
   	xstep=(x2-x1)/dble(nbod1-1)
        ystep=(y2-y1)/dble(nbod2-1)
        do i=1,nbod1
          x(i)=x1+xstep*dble(i-1)
	enddo
        do i=1,nbod2
          y(i)=y1+ystep*dble(i-1)
        enddo
	call roche(rosu,rotem,areas,teff
     p  ,rfront,rback,rpole,rside,rmean	
     p  ,tstar,rstar,dgst,irrst,albst,htst,htsta
     p  ,tempcp,rcp,dgcp,irrcp,albcp,htcp,htcpa
     p  ,x,y,xcomp,q,ff,nbod1,nbod2,ipsc)
c	write(*,*)'Roche surface =',areas
     	write(*,*)'effective temp [K] =',teff
	rfront=rfront*xcomp/rjup
	rback=rback*xcomp/rjup
	rpole=rpole*xcomp/rjup
	rside=rside*xcomp/rjup
	rmean=rmean*xcomp/rjup
        write(*,*)'   Rsub         Rback        Rpole        Rside'
        write(*,'(4f13.8)')rfront,rback,rpole,rside
        write(*,*)'   Rmean        Rcross       ff'
        write(*,'(3f13.8)')rmean,dsqrt(rpole*rside),ff
        write(*,*)'   Rsub/Rside   Rsub/Rpole   Rside/Rpole'
        write(*,'(3f13.8)')rfront/rside,rfront/rpole,rside/rpole
	stop
	end
c-----------------------------------------------------------------------
	subroutine roche(rosu,rotem,areas,teff
     p  ,rfront,rback,rpole,rside,rmean	
     p  ,tstar,rstar,dgst,irrst,albst,htst,htsta
     p  ,tempcp,rcp,dgcp,irrcp,albcp,htcp,htcpa
     p  ,x,y,xcomp,q,ff,nbod1,nbod2,ipsc)
c	Subroutine calculates the Roche surface (RS), normalized 
c       gravity on RS of the primary, secondary or contact system,
c	irradiation angle, and temperature on RS including gravity 
c	darkening and an approximate irradiation effect with
c	reflection, heating, and heat redistribution for detached 
c	components.
c	Code works in normalized scale where center of 
c	mass of the primary is at x=0, center of mass of the secondary 
c	at x=1 but the input/output is scaled/unscaled. 
c	CGS units are employed.
c		input: 
c	x,y -grid points in orbital plane
c	nbod1,nbod2 -number of grid points in x and y
c	xcomp -is the scale=x coordinate of the secondary 
c	q -mass ratio (q<1 -more massive star is in the center,
c		q>1 -less massive star is in the center)
c	ff -radius of the planet, later it becomes:
c          Roche lobe filling factor of the primary (if ipsc=1). It is
c	   the distance of the inner substellar point of the primary 
c	   (between the stars) from the center of the primary relative 
c	   to the distance to L1, 
c	   ff<=1, the Roche lobe is reproduced if ff=1 
c	ff -Roche lobe filling factor of the secondary (if ipsc=2).
c	   It is the radius of the secondary at the substellar point 
c	   relative to 1-L1, 
c	   ff<=1, the Roche lobe is reproduced if ff=1 
c	ff -Roche lobe fill-out factor of the contact system (if ipsc=3)
c		ff=(P1-P)/(P1-P2)+1   where P,P1,P2 are potentials
c	ipsc=1 -calculates RS of a detached central star (primary)
c	ipsc=2 -calculates RS of a detached secondary
c	ipsc=3 -calculates RS of a contact system
c       tstar -effective temperature of the central star in [K]      
c          in the absence of irradiation effect
c          if ipsc=1 it is the temperature at the rotation pole 
c          if ipsc=3 it is the temperature at the rotation pole of
c          the more massive star
c       rstar -radius of the central star used for the irradiation 
c		of other object
c       dgst -gravity darkening coefficient (beta) of the central star
c               (0.25 for radiative, 0.08 for convective atmospheres) 
c       irrst=0 -irradiation (reflection effect) is off
c           (albstp,htst,htsta have no meaning in this case)
c       irrst=1 -irradiation of the object from the companion is on.
c       albst  -Bond albedo <0,1> of the central star
c       htst   -heat redistribution parameter in case of the irradiation, 
c           <0,1>, 0-nothing is redistributed over the day side and 
c          nothing goes to the night, 1-all the heat is evenly 
c          redistributed over the day and night sides 
c       htsta  -zonal temperature redistribution parameter i.e.
c	   a degree of the homegenity of the heat transport  
c           <0,1>. 1-homegeneous, 0-cosine dependence (zonal)
c           T**4=T0**4(htsta+(1-htsta)*cos_latitude*constant)
c       tempcp -effective temperature at the rotation pole of 
c	   the secondary in [K] in the absence of irradiation effect
c       rcp -radius of the secondary used for the irradiation
c               of other object
c	irrcp,albcp,htcp,htcpa -the same as in the case of primary
c
c		output: 
c	rosu(x,y) -Roche surface
c	rotem(x,y) -surface temperature
c       areas -area of the RS
c	teff -effective temperature of the object including irradiation
c		and gravity darkening
c 	rfront,rback,rpole,rside,rmean -radius relative to the semimajor
c		axis at the substellar point, antistellar point, 
c		rotation pole, side point, and radius of a sphere with
c		 the same volume
c	rogrv(x,y) -normalized gravity on the RS relative to the gravity
c		on the rotation pole (of more massive star in case of
c		contact system)
c	rocos(x,y) -cosine of the angle between gravity vector
c		(-1*normal to the surface) and direction towards
c		the irradiating star i.e. cosine of the zenit 
c		distance of the irradiating star as seen from 
c		the irradiated surface). It is used for 
c		an approximate reflection effect (only for ipsc=1,2). 
c		rocos=0 on the night side.
c
c       Author/contact: Jan Budaj, http://www.ta3.sk/~budaj
c                       budaj@ta3.sk
c
	implicit double precision (a-h,o-z)
c 	include 'param.inc'
	parameter(ndim1=2001,ndim2=2001)
	dimension x(ndim1),y(ndim2),xn(ndim1),yn(ndim2)
     p  ,rosuy(ndim1),rosu(ndim1,ndim2),rogrv(ndim1,ndim2)
     p  ,rocos(ndim1,ndim2),rotem(ndim1,ndim2)
     p  ,area(ndim1,ndim2),areaz(ndim1,ndim2),arear(ndim1,ndim2)
	external pot,dxpot,dypot,dzpot,dxxpot
	pi=3.1415926535897931d0
	stefb=5.670400d-5
c	if((ipsc.eq.1.or.ipsc.eq.2).and.(ff.gt.1.d0.or.ff.lt.1.d-2))
c     p  then
c	  write(*,130)
c	  goto 120
c	endif	
	if(ipsc.eq.3.and.(ff.le.1.d0.or.ff.gt.2.d0))then
	  write(*,130)
	  goto 120
	endif
	if(.not.(ipsc.eq.1.or.ipsc.eq.2.or.ipsc.eq.3))then
	  write(*,130)
	  goto 120
	endif
	if(htst.lt.0.d0.or.htst.gt.1.d0.or.
     p	  htsta.lt.0.d0.or.htsta.gt.1.d0.or.
     p	  htcp.lt.0.d0.or.htcp.gt.1.d0.or.
     p	  htcpa.lt.0.d0.or.htcpa.gt.1.d0)then
	  write(*,130)
	  goto 120
	endif	
c	precision required
	prc=1.d-7
	prc2=1.d-9
c	normalization of coordinates
        do 10 i=1,nbod1
          xn(i)=x(i)/xcomp
10      continue
        do 20 i=1,nbod2
          yn(i)=y(i)/xcomp
20      continue
c		el1- the x coordinate of the L1 point 
c	it may be quicker to start with 0.5 then with radlob function
	el1=raphx(dxpot,dxxpot,0.d0,q,0.5d0,prc2)
c       	el2,el3 -the L2,L3 points (they are reversed if q>1)
        el2=raphx(dxpot,dxxpot,0.d0,q,2.d0-el1,prc2)
        el3=raphx(dxpot,dxxpot,0.d0,q,-el1,prc2)
c
c??	temporary settings for ipsc=1, ff is radius of the planet [Rjup]
        rjup=7.149e9
	if(ipsc.eq.1)then
	  ff=ff*rjup/xcomp/el1
	endif
	if((ipsc.eq.1.or.ipsc.eq.2).and.(ff.gt.1.d0.or.ff.lt.1.d-2))
     p  then
	  write(*,130)
	  goto 120
	endif	
c??	ff is fill-in parameter
c			primary, ff<=1
	if(ipsc.eq.1.and.ff.gt.0.d0.and.ff.le.1.d0)then
	  pinn=ff*el1
	  pout=-pinn
	  rpot=pot(pinn,0.d0,0.d0,q)
c	  pinn-the inner substellar point of the surface of the primary
c	  pout-the outer point of the primary on the averted hemisphere
	  if(pout.gt.el3)then
   	    pout=pout
	  else
	    pout=el3*0.5d0
	  endif
	  pout=raphx(pot,dxpot,rpot,q,pout,prc2)
c	  rosuy(x)-the shape of the primary Roche surface for x=x(i),z=0
c	  ystart=el1*0.2d0
c	  expression below might help for small ff
	  ystart=pinn*0.5d0	 
	  do 50 i=1,nbod1
	    if(xn(i).gt.pout.and.xn(i).lt.pinn)then
	      rosuy(i)=dabs(raphy(pot,dypot,rpot,q,xn(i),ystart,prc))
	      ystart=rosuy(i)
	    else
	      rosuy(i)=0.d0
	    endif
50	  continue
c	  coordinates of the rotation pole
	  xpole=0.d0
	  ypole=0.d0
c	  zpole=raphz(pot,dzpot,rpot,q,xpole,ypole,radlob(1.d0/q),prc2)
	  zpole=raphz(pot,dzpot,rpot,q,xpole,ypole,dabs(pout),prc2)	  
	  rfront=pinn
	  rback=-pout
	  rpole=zpole
	  rside=dabs(raphy(pot,dypot,rpot,q,0.d0,pout,prc2))
	endif
c
c			contact, 1<ff<=2
	if(ipsc.eq.3.and.ff.gt.1.d0.and.ff.le.2.d0)then
	  potel1=pot(el1,0.d0,0.d0,q)	
	  potel2=pot(el2,0.d0,0.d0,q)	
	  potel3=pot(el3,0.d0,0.d0,q)	
	  if(potel2.lt.potel3)then
	    pom=potel2
	    potel2=potel3
	    potel3=pom
	  endif
	  rpot=potel1-(ff-1.d0)*(potel1-potel2)
	  pout=raphx(pot,dxpot,rpot,q,el3*0.5d0,prc2)
	  pinn=raphx(pot,dxpot,rpot,q,0.5d0+el2*0.5d0,prc2)
c	  pinn-the right hand side edge of the contact Roche surface
c	  pout-the left hand side edge of the contact Roche surface
c	  rosuy(x)-the shape of the contact Roche surface for x=x(i),z=0
	  ystart=radlob(1.d0/q)*0.2d0
	  do 70 i=1,nbod1
	    if(xn(i).gt.pout.and.xn(i).lt.pinn)then
	      rosuy(i)=dabs(raphy(pot,dypot,rpot,q,xn(i),ystart,prc))
	      ystart=rosuy(i)	
	    else
	      rosuy(i)=0.d0
	    endif
c	    write(3,*)xn(i),rosuy(i),pout
70	  continue
c	  coordinates of the rotation pole of the more massive star
	  if(q.lt.1.d0)then
	  xpole=0.d0
	  ypole=0.d0
	  zpole=raphz(pot,dzpot,rpot,q,xpole,ypole,radlob(1.d0/q),prc2)
	  rside=dabs(raphy(pot,dypot,rpot,q,xpole,radlob(1.d0/q),prc2))	  	  
	  else
	  xpole=1.d0
	  ypole=0.d0
	  zpole=raphz(pot,dzpot,rpot,q,xpole,ypole,radlob(q),prc2)
	  rside=dabs(raphy(pot,dypot,rpot,q,xpole,radlob(q),prc2))	  
	  endif
	  rfront=pinn
	  rback=-pout
	  rpole=zpole
	endif
c
c			secondary, ff<=1
	if(ipsc.eq.2.and.ff.gt.0.d0.and.ff.le.1.d0)then
	  sout=ff*(1.d0-el1)
	  sinn=1.d0-sout
	  rpot=pot(sinn,0.d0,0.d0,q)
c	  sinn-inner substellar point of the surface of the secondary
c	  sout-outer point of the secondary on the averted hemisphere
	  if((1.d0+sout).lt.el2)then
   	    sout=1.d0+sout
	  else
	    sout=1.d0+(el2-1.d0)*0.5d0
	  endif
	  sout=raphx(pot,dxpot,rpot,q,sout,prc2)
c	  rosuy(x)-the shape of the Roche surface for x=x(i),z=0
	  ystart=(sout-1.d0)*0.2d0
	  do 90 i=1,nbod1
	    if(xn(i).gt.sinn.and.xn(i).lt.sout)then
	      rosuy(i)=dabs(raphy(pot,dypot,rpot,q,xn(i),ystart,prc))
	      ystart=rosuy(i)
	    else
	      rosuy(i)=0.d0
	    endif
90	  continue
c	  coordinates of the rotation pole
	  xpole=1.d0
	  ypole=0.d0
	  zpole=radlob(q)*ff
	  zpole=raphz(pot,dzpot,rpot,q,xpole,ypole,zpole,prc2)
	  rfront=1.d0-sinn
	  rback=sout-1.d0
	  rpole=zpole
	  rside=dabs(raphy(pot,dypot,rpot,q,1.d0,zpole,prc2))
	endif
c	check mainly this
  	do 95 i=1,nbod1
	  if(rosuy(i).gt.0.d0.and.
     p    dypot(xn(i),rosuy(i),0.d0,q).ge.0.d0)then
	    write(*,*)' Error: Newton-Raphson slipped in subr.: Roche,',
     p      ' try to modify slightly: nbod,q,ff'
	  goto 120
	  endif
95	continue
c
c	gravnp-normalized gravity at the rotation pole
	call gravn(xpole,ypole,zpole,q,potx,poty,potz,gravnp)
c
c		rosu(x,y)-the shape of the Roche surface
c		rogrv(x,y) -normalized gravity relative to the pole
c		rocos(i,j) -cosine of the irradiation angle
	do  i=1,nbod1
          do  j=1,nbod2
            if(dabs(yn(j)).lt.rosuy(i))then
	      rosu(i,j)=raphz(pot,dzpot,rpot,q,xn(i),yn(j),rosuy(i)
     p        ,prc)
	      call gravn(xn(i),yn(j),rosu(i,j),q,potx,poty,potz,gravec)
	      rogrv(i,j)=gravec/gravnp
	      areaz(i,j)=dabs(potz/gravec)
	      if(ipsc.eq.1)then
	        rvec=dsqrt((xn(i)-1.d0)**2+yn(j)**2+rosu(i,j)**2)
	        rocos(i,j)=((xn(i)-1.d0)*potx+yn(j)*poty+rosu(i,j)*potz)
     p	        /rvec/gravec
       	        if(rogrv(i,j).lt.1.d-4)then
	          rogrv(i,j)=1.d-4	
	          rocos(i,j)=0.84d0
	        endif
		rotem(i,j)=tstar*rogrv(i,j)**dgst
	      elseif(ipsc.eq.2)then
	        rvec=dsqrt(xn(i)*xn(i)+yn(j)*yn(j)+rosu(i,j)*rosu(i,j))
	        rocos(i,j)=(xn(i)*potx+yn(j)*poty+rosu(i,j)*potz)/rvec
     p	        /gravec
c		take care of the unimportant case of singularity
     	        if(rogrv(i,j).lt.1.d-4)then
	          rogrv(i,j)=1.d-4	
	          rocos(i,j)=0.84d0
	        endif
		rotem(i,j)=tempcp*rogrv(i,j)**dgcp
              else
	        if(rogrv(i,j).lt.1.d-4)rogrv(i,j)=1.d-4	
                rocos(i,j)=0.d0
		rotem(i,j)=tstar*rogrv(i,j)**dgst
	      endif	 
    	      if(rocos(i,j).lt.0.d0)rocos(i,j)=0.d0	
            else
	      rosu(i,j)=0.d0
	      rogrv(i,j)=0.d0
	      rocos(i,j)=0.d0
	      rotem(i,j)=0.d0
	      areaz(i,j)=0.d0
            endif
c		unscale the result
	    rosu(i,j)=xcomp*rosu(i,j)
	  enddo
	enddo
c	
c		irradiation and heat redistribution
	call surf(ndim1,ndim2,nbod1,nbod2,x,y,area)
c	area-projection of the surface element to xy
c	arear-area of the surface element
c	vol-volume under the Roche surface
        areas=0.d0
	vol=0.d0
        sum1=0.d0
        sum2=0.d0        
        do i=1,nbod1  
          do j=1,nbod2
            if(dabs(yn(j)).lt.rosuy(i))then  
              arear(i,j)=area(i,j)/areaz(i,j)
              if(irrst.eq.1.and.ipsc.eq.1)then
                fxy2=x(i)**2+y(j)**2
                coslat=fxy2/(fxy2+rosu(i,j)**2)
                coslat=dsqrt(coslat)
                surf2=coslat*arear(i,j)
                if(rocos(i,j).gt.0.d0)then
c                 dayside
                  rr2=rcp**2/((x(i)-xcomp)**2+y(j)**2+rosu(i,j)**2)
                  firr=stefb*rr2*tempcp**4*rocos(i,j)
		  surf1=firr*arear(i,j)
                endif
              endif
              if(irrcp.eq.1.and.ipsc.eq.2)then
                fxy2=(x(i)-xcomp)**2+y(j)**2
                coslat=fxy2/(fxy2+rosu(i,j)**2)
                coslat=dsqrt(coslat)
                surf2=coslat*arear(i,j)
                if(rocos(i,j).gt.0.d0)then
c                 day-side
                  rr2=rstar*rstar/(x(i)**2+y(j)**2+rosu(i,j)**2)
                  firr=stefb*rr2*tstar**4*rocos(i,j)
                  surf1=firr*arear(i,j)
                endif    
              endif              
              areas=areas+arear(i,j)
              vol=vol+area(i,j)*rosu(i,j)
	      sum1=sum1+surf1
	      sum2=sum2+surf2
            endif
          enddo
        enddo
        areas=areas*2.d0
        rmean=(vol*2.d0/(4.d0/3.d0*pi))**(1.d0/3.d0)/xcomp
        sum1=sum1*2.d0
        sum2=sum2*2.d0
        if(irrst.eq.1.and.ipsc.eq.1)then
          rr2=rcp**2/xcomp**2
          t04=0.25d0*htst*(1.d0-albst)*rr2*tempcp**4
          t04new=htst*(1.d0-albst)*sum1/areas/stefb
             t04=htst*(1.d0-albst)*sum1/areas/stefb          
          htb=4.d0*(1.d0-htsta)/pi
          htbnew=(1.d0-htsta)*areas/sum2
             htb=(1.d0-htsta)*areas/sum2          
        endif  
        if(irrcp.eq.1.and.ipsc.eq.2)then
          rr2=rstar*rstar/xcomp**2  
          t04=0.25d0*htcp*(1.d0-albcp)*rr2*tstar**4            
          t04new=htcp*(1.d0-albcp)*sum1/areas/stefb
c            t04=htcp*(1.d0-albcp)*sum1/areas/stefb          
          htb=4.d0*(1.d0-htcpa)/pi
          htbnew=(1.d0-htcpa)*areas/sum2
c            htb=(1.d0-htcpa)*areas/sum2          
        endif  
c	sum1,t04new may not be precise enough
c	(polar or cylindrical coordinates might be better).
c	However, if you want to use them simply uncomment 
c	the four lines above and replace
c	t04 by t04new and htb by htbnew.
c	write(*,*)'T0=',t04new**0.25d0,t04**0.25d0
c	write(*,*)'b=',htbnew,htb
c	write(*,*)'sum1,2=',sum1,sum2
	teff=0.d0
	do i=1,nbod1
          do j=1,nbod2
            if(dabs(yn(j)).lt.rosuy(i))then
              if(irrst.eq.1.and.ipsc.eq.1)then
                fxy2=x(i)**2+y(j)**2
                coslat=fxy2/(fxy2+rosu(i,j)**2)
                coslat=dsqrt(coslat)
                fnlat=htsta+htb*coslat
                if(rocos(i,j).gt.0.d0)then
c                 dayside
                  rr2=rcp**2/((x(i)-xcomp)**2+y(j)**2+rosu(i,j)**2)
                  firr=stefb*rr2*tempcp**4*rocos(i,j)
                  tirr4=(1.d0-htst)*(1.d0-albst)*firr/stefb
                  rotem(i,j)=(tirr4+t04*fnlat+rotem(i,j)**4)
		  rotem(i,j)=dsqrt(dsqrt(rotem(i,j)))	
                else 
c                 nightside             
                  rotem(i,j)=(t04*fnlat+rotem(i,j)**4)
                  rotem(i,j)=dsqrt(dsqrt(rotem(i,j)))
                endif
              endif
              if(irrcp.eq.1.and.ipsc.eq.2)then
                fxy2=(x(i)-xcomp)**2+y(j)**2
                coslat=fxy2/(fxy2+rosu(i,j)**2)
                coslat=dsqrt(coslat)
                fnlat=htcpa+htb*coslat  
                if(rocos(i,j).gt.0.d0)then
c                 day-side
                  rr2=rstar*rstar/(x(i)**2+y(j)**2+rosu(i,j)**2)
                  firr=stefb*rr2*tstar**4*rocos(i,j)
                  tirr4=(1.d0-htcp)*(1.d0-albcp)*firr/stefb
                  rotem(i,j)=(tirr4+t04*fnlat+rotem(i,j)**4)
                  rotem(i,j)=dsqrt(dsqrt(rotem(i,j)))
                else
c                 night-side
                  rotem(i,j)=(t04*fnlat+rotem(i,j)**4)
		  rotem(i,j)=dsqrt(dsqrt(rotem(i,j)))
                endif    
              endif
              teff=teff+arear(i,j)*rotem(i,j)**4
            endif
	  enddo
	enddo
	teff=(teff*2.d0/areas)**0.25d0
c
     	open(4,file='roche.out',status='unknown')
	do i=1,nbod1
          do j=1,nbod2
            write(4,180)x(i),y(j),rosu(i,j),rogrv(i,j),rocos(i,j)
     p      ,rotem(i,j)
	  enddo
	  write(4,*)
	enddo
180	format(6e14.5)
	close(4)	
c
120	return
130	format(' error: inconsistent input in subroutine: roche')
	end
c-----------------------------------------------------------------------
        double precision function raphx(fun,dfun,fun0,q,xstart,xacc)
c	Newton-Raphson Method in 1D (x coordinate)
c	to solve the equation fun(x)=fun0
c	input:  fun-external function of x and parameters: y,z,q
c		dfun-external function, partial derivative df/dx
c		fun0-value of the function 
c		q-mass ratio
c	        xstart-first estimate
c		xacc-accuracy
c	output:  raphx-root for which fun=fun0
        implicit double precision (a-h,o-z)
	external fun,dfun
	raphx=xstart
	do i=1,22
	  dx=(fun(raphx,0.d0,0.d0,q)-fun0)/dfun(raphx,0.d0,0.d0,q)
	  raphx=raphx-dx
	  if(dabs(dx).lt.xacc)return
 	enddo
	return
	end
c-----------------------------------------------------------------------
        double precision function raphy(fun,dfun,fun0,q,x0
     p  ,ystart,yacc)
c	Newton-Raphson Method in 1D (y coordinate)
c	to solve the equation fun(y)=fun0
c	input:  fun-external function of y and parameters: x,z,q
c		dfun-external function, partial derivative df/dy
c		fun0-value of the function 
c		q-mass ratio
c	        ystart-first estimate
c		yacc-accuracy
c	output:  raphy-root for which fun=fun0
        implicit double precision (a-h,o-z)
	external fun,dfun
	raphy=ystart
	do i=1,22
	  dy=(fun(x0,raphy,0.d0,q)-fun0)/dfun(x0,raphy,0.d0,q)
	  raphy=raphy-dy
	  if(dabs(dy).lt.yacc)return
 	enddo
	return
	end
c-----------------------------------------------------------------------
        double precision function raphz(fun,dfun,fun0,q,x0,y0
     p  ,zstart,zacc)
c	Newton-Raphson Method in 1D (z coordinate)
c	to solve the equation fun(z)=fun0
c	input:  fun-external function of z and parameters: x,y,q
c		dfun-external function, partial derivative df/dz
c		fun0-value of the function 
c		q-mass ratio
c	        zstart-first estimate
c		zacc-accuracy
c	output:  raphz-root for which fun=fun0
        implicit double precision (a-h,o-z)
	external fun,dfun
	raphz=zstart
	do i=1,22
	  dz=(fun(x0,y0,raphz,q)-fun0)/dfun(x0,y0,raphz,q)
	  raphz=raphz-dz
	  if(dabs(dz).lt.zacc)return
 	enddo
	return
	end
c-----------------------------------------------------------------------
	double precision function pot(x,y,z,q)
c	input: x,y,z,q
c	output: normalized Roche potential 
        implicit double precision (a-h,o-z)
	q1=1.d0+q
	x1=x-1.d0
	r1=dsqrt(x*x+y*y+z*z)
	r2=dsqrt(x1*x1+y*y+z*z)
	pot=2.d0/q1/r1+2.d0*q/q1/r2+(x-q/q1)*(x-q/q1)+y*y
	return
	end
c-----------------------------------------------------------------------
	double precision function dxpot(x,y,z,q)
c	input: x,y,z,q
c	output: partial derivation of the Roche potential after x
        implicit double precision (a-h,o-z)
	q1=1.d0+q
	x1=x-1.d0
	r1=dsqrt(x*x+y*y+z*z)
	r2=dsqrt(x1*x1+y*y+z*z)
	dxpot=-x/q1/r1**3-q*x1/q1/r2**3+x-q/q1
	dxpot=2.d0*dxpot
	return
	end
c-----------------------------------------------------------------------
	double precision function dypot(x,y,z,q)
c	input: x,y,z,q
c	output: partial derivation of the Roche potential after y
        implicit double precision (a-h,o-z)
	q1=1.d0+q
	x1=x-1.d0
	r1=dsqrt(x*x+y*y+z*z)
	r2=dsqrt(x1*x1+y*y+z*z)
	dypot=-2.d0/q1/r1**3-2.d0*q/q1/r2**3+2.d0
	dypot=dypot*y
	return
	end
c-----------------------------------------------------------------------
	double precision function dzpot(x,y,z,q)
c	input: x,y,z,q
c	output: partial derivation of the Roche potential after z
        implicit double precision (a-h,o-z)
	q1=1.d0+q
	x1=x-1.d0
	r1=dsqrt(x*x+y*y+z*z)
	r2=dsqrt(x1*x1+y*y+z*z)
	dzpot=-2.d0/q1/r1**3-2.d0*q/q1/r2**3
	dzpot=dzpot*z
	return
	end
c-----------------------------------------------------------------------
	double precision function dxxpot(x,y,z,q)
c	input: x,y,z,q
c	output: second partial derivation of the Roche potential after x
        implicit double precision (a-h,o-z)
	q1=1.d0+q
	x1=x-1.d0
	r1=dsqrt(x*x+y*y+z*z)
	r2=dsqrt(x1*x1+y*y+z*z)
	dxxpot=6.d0*x*x/q1/r1**5+6.d0*q*x1*x1/q1/r2**5-2.d0/q1/r1**3
     p  -2.d0*q/q1/r2**3+2.d0
	return
	end
c-----------------------------------------------------------------------
	double precision function radlob(q)
c	returns the volumee radius of the Roche lobe, 0<q<infty
c	q<1 -radius of the less massive star
c	q>1 -radius of the more massive star
        implicit double precision (a-h,o-z)
	q13=q**(1.d0/3.d0)
	radlob=0.49d0*q13*q13/(0.69d0*q13*q13+dlog(1.d0+q13))
	return
	end
c-----------------------------------------------------------------------
        subroutine gravn(x,y,z,q,potx,poty,potz,pot)
c       calculates the normalized gravity vector
c       input: x,y,z,q
c       output: potx,poty,potz,pot
        implicit double precision (a-h,o-z)
        potx=dxpot(x,y,z,q)
        poty=dypot(x,y,z,q)
        potz=dzpot(x,y,z,q)
        pot=dsqrt(potx*potx+poty*poty+potz*potz)
        return
        end
c-----------------------------------------------------------------------
	subroutine surf(ndim1,ndim2,nbod1,nbod2,ar,at,area)
c	to calculate area(i,j)-area of a projected surface element
c	input: ar,at
c	output: area
	implicit double precision (a-h,o-z)
	dimension ar(ndim1),at(ndim2),area(ndim1,ndim2)
	pi=3.1415926535897931d0
	do i=2,nbod1-1
	do j=2,nbod2-1
	  area(i,j)=(ar(i+1)-ar(i-1))*(at(j+1)-at(j-1))*0.25d0
	enddo
	enddo
	pomi12=(ar(2)-ar(1))*0.5d0
	pominn=(ar(nbod1)-ar(nbod1-1))*0.5d0
	do j=2,nbod2-1
	  area(1,j)=pomi12*(at(j+1)-at(j-1))*0.5d0
	  area(nbod1,j)=pominn*(at(j+1)-at(j-1))*0.5d0
	enddo
	do i=2,nbod1-1
	  area(i,1)=(ar(i+1)-ar(i-1))*(at(2)-at(1))*0.25d0
	  area(i,nbod2)=(ar(i+1)-ar(i-1))*(at(nbod2)-at(nbod2-1))*
     p    0.25d0
	enddo
	area(1,1)=pomi12*(at(2)-at(1))*0.5d0
        area(1,nbod2)=pomi12*(at(nbod2)-at(nbod2-1))*0.5d0
        area(nbod1,1)=pominn*(at(2)-at(1))*0.5d0
        area(nbod1,nbod2)=pominn*(at(nbod2)-at(nbod2-1))*0.5d0
	return
	end
